@@ -88,13 +88,11 @@ DESeq2BIC=function(data,name="BIC",mode="total",normalization=mode,formulas=list
 #' This can be accomplished by setting mode to "new", and normalization to "total"!
 #'
 #' @return a new grandR object including a new analysis table. The columns of the new analysis table are
-#' \itemize{
 #'  \item{"M"}{the base mean}
 #'  \item{"S"}{the difference in deviance between the reduced model and the full model}
 #'  \item{"P"}{the likelihood ratio test P value}
 #'  \item{"Q"}{same as P but Benjamini-Hochberg multiple testing corrected}
 #'  \item{"LFC"}{the log2 fold change for the target model (only with the logFC parameter set to TRUE)}
-#' }
 #'
 #' @export
 #'
@@ -157,6 +155,7 @@ LikelihoodRatioTest=function(data,name="LRT",mode="total",slot="count",normaliza
 #' @param contrasts contrast matrix that defines all pairwise comparisons, generated using \link{GetContrasts}
 #' @param mode.slot which slot to take expression values from
 #' @param FUN a function taking 1. the data matrix, 2. a logical vector indicating condition A and 3. a logical vector indicating condition B
+#' @param genes restrict analysis to these genes; NULL means all genes
 #' @param verbose print status messages?
 #' @param ... further parameters forward to FUN
 #'
@@ -169,15 +168,15 @@ LikelihoodRatioTest=function(data,name="LRT",mode="total",slot="count",normaliza
 #' @export
 #'
 #' @concept helper
-ApplyContrasts=function(data,analysis,name.prefix,contrasts,mode.slot=NULL,verbose=FALSE,FUN,...) {
+ApplyContrasts=function(data,analysis,name.prefix,contrasts,mode.slot=NULL,genes=NULL,verbose=FALSE,FUN,...) {
   if (is.null(mode.slot)) stop("Need to specify mode.slot!")
 
-  mat=as.matrix(GetTable(data,type=mode.slot,ntr.na=FALSE))
+  mat=as.matrix(GetTable(data,type=mode.slot,ntr.na=FALSE,genes = genes))
   mode.slot=get.mode.slot(data,mode.slot)
   for (n in names(contrasts)) {
     if (verbose) cat(sprintf("Computing %s for %s...\n",analysis,n))
     re.df=FUN(mat,contrasts[[n]]==1,contrasts[[n]]==-1,...)
-    data=AddAnalysis(data,name = if (is.null(name.prefix)) n else paste0(name.prefix,".",n),table = re.df)
+    data=AddAnalysis(data,name = if (is.null(name.prefix)) n else paste0(name.prefix,".",n),table = re.df,warn.genes = is.null(genes))
   }
   data
 }
@@ -194,36 +193,38 @@ ApplyContrasts=function(data,analysis,name.prefix,contrasts,mode.slot=NULL,verbo
 #' @param slot the slot of the grandR object to take the data from; should contain counts!
 #' @param mode compute LFCs for "total", "new", or "old" RNA
 #' @param normalization normalize on "total", "new", or "old" (see details)
+#' @param genes restrict analysis to these genes; NULL means all genes
 #' @param verbose print status messages?
 #'
 #' @details Both \link[lfc]{PsiLFC} and  \link[lfc]{NormLFC}) by default perform normalization by subtracting the median log2 fold change from all log2 fold changes.
 #' When computing LFCs of new RNA, it might be sensible to normalize w.r.t. to total RNA, i.e. subtract the median log2 fold change of total RNA from all the log2 fold change of new RNA.
 #' This can be accomplished by setting mode to "new", and normalization to "total"!
-
+#'
 #' @details Normalization can also be a mode.slot! Importantly, do not specify a slot containing normalized values, but specify a slot of unnormalized values
-#' (which are used to compute the size factors for normalization!)
+#' (which are used to compute the size factors for normalization!) Can also be a numeric vector of size factors with the same length as the data as columns.
+#' Then each value is divided by the corresponding size factor entry.
 #'
 #' @return a new grandR object including a new analysis table. The columns of the new analysis table are
-#' \itemize{
 #'  \item{"M"}{the base mean}
 #'  \item{"S"}{the log2FoldChange divided by lfcSE}
 #'  \item{"P"}{the Wald test P value}
 #'  \item{"Q"}{same as P but Benjamini-Hochberg multiple testing corrected}
 #'  \item{"LFC"}{the log2 fold change}
-#' }
+#'
 #' @seealso \link{PairwiseDESeq2},\link{GetContrasts}
 #' @export
 #'
 #' @concept diffexp
 Pairwise=function(data, name.prefix = mode, contrasts, LFC.fun=lfc::PsiLFC, slot="count", mode="total",
                normalization=mode,
+               genes = NULL,
                verbose=FALSE) {
 
   contrasts = contrasts[,apply(contrasts,2,function(v) all(c(-1,1) %in% v)),drop=FALSE]
   if (ncol(contrasts)==0) stop("Contrasts do not define any comparison!")
 
-  data=LFC(data,name.prefix = name.prefix, contrasts = contrasts, LFC.fun = LFC.fun, slot=slot, mode=mode,normalization = normalization,verbose=verbose)
-  data=PairwiseDESeq2(data,name.prefix = name.prefix, contrasts = contrasts, slot=slot, mode=mode,normalization = normalization,verbose=verbose)
+  data=LFC(data,name.prefix = name.prefix, contrasts = contrasts, LFC.fun = LFC.fun, slot=slot, mode=mode,normalization = normalization,compute.M = FALSE,genes=genes,verbose=verbose)
+  data=PairwiseDESeq2(data,name.prefix = name.prefix, contrasts = contrasts, slot=slot, mode=mode,normalization = normalization,genes=genes,verbose=verbose)
   data
 
 }
@@ -239,6 +240,8 @@ Pairwise=function(data, name.prefix = mode, contrasts, LFC.fun=lfc::PsiLFC, slot
 #' @param LFC.fun function to compute log fold changes (default: \link[lfc]{PsiLFC}, other viable option: \link[lfc]{NormLFC})
 #' @param mode compute LFCs for "total", "new", or "old" RNA
 #' @param normalization normalize on "total", "new", or "old" (see details)
+#' @param compute.M also compute the mean expression (in log10 space)
+#' @param genes restrict analysis to these genes; NULL means all genes
 #' @param verbose print status messages?
 #' @param ... further arguments forwarded to LFC.fun
 #'
@@ -247,12 +250,11 @@ Pairwise=function(data, name.prefix = mode, contrasts, LFC.fun=lfc::PsiLFC, slot
 #' This can be accomplished by setting mode to "new", and normalization to "total"!
 #'
 #' @details Normalization can also be a mode.slot! Importantly, do not specify a slot containing normalized values, but specify a slot of unnormalized values
-#' (which are used to compute the size factors for normalization!)
+#' (which are used to compute the size factors for normalization!) Can also be a numeric vector of size factors with the same length as the data as columns.
+#' Then each value is divided by the corresponding size factor entry.
 #'
 #' @return a new grandR object including a new analysis table. The columns of the new analysis table are
-#' \itemize{
 #'  \item{"LFC"}{the log2 fold change}
-#' }
 #'
 #' @seealso \link{PairwiseDESeq2},\link{GetContrasts}
 #' @export
@@ -269,6 +271,8 @@ Pairwise=function(data, name.prefix = mode, contrasts, LFC.fun=lfc::PsiLFC, slot
 #' @concept diffexp
 LFC=function(data, name.prefix = mode, contrasts, slot="count",LFC.fun=lfc::PsiLFC, mode="total",
              normalization=NULL,
+             compute.M=TRUE,
+             genes=NULL,
              verbose=FALSE,...) {
 
   contrasts = contrasts[,apply(contrasts,2,function(v) all(c(-1,1) %in% v)),drop=FALSE]
@@ -276,20 +280,31 @@ LFC=function(data, name.prefix = mode, contrasts, slot="count",LFC.fun=lfc::PsiL
 
   mode.slot=paste0(mode,".",slot)
   if (!is.null(normalization)) {
-    if (!check.mode.slot(data,normalization)) normalization=paste0(normalization,".",slot)
-    if (!check.mode.slot(data,normalization)) stop("Invalid normalization!")
+    if (is.numeric(normalization)) {
+      if (length(normalization)!=ncol(data)) stop("Invalid normalization, for size factors make sure that it matches the columns of data!")
+    } else {
+      if (!check.mode.slot(data,normalization)) normalization=paste0(normalization,".",slot)
+      if (!check.mode.slot(data,normalization)) stop("Invalid normalization!")
+    }
   }
   if (!check.mode.slot(data,mode.slot)) stop("Invalid mode")
-  ApplyContrasts(data,name.prefix=name.prefix,contrasts=contrasts,mode.slot=mode.slot,verbose=verbose,analysis="LFC",FUN=function(mat,A,B) {
+  ApplyContrasts(data,name.prefix=name.prefix,contrasts=contrasts,mode.slot=mode.slot,genes=genes,verbose=verbose,analysis="LFC",FUN=function(mat,A,B) {
     if (!is.null(normalization)) {
-      norm.mat=as.matrix(GetTable(data,type=normalization,ntr.na=FALSE))
-      nlfcs=LFC.fun(rowSums(norm.mat[,A,drop=FALSE]),rowSums(norm.mat[,B,drop=FALSE]),normalizeFun=function(i) i)
-      med.element=median(nlfcs)
-      lfcs=LFC.fun(rowSums(mat[,A,drop=FALSE]),rowSums(mat[,B,drop=FALSE]),normalizeFun=function(i) i-med.element,...)
+      if (is.numeric(normalization)) {
+        shift=log2(sum(normalization[A])/sum(normalization[B]))
+        lfcs=LFC.fun(rowSums(mat[,A,drop=FALSE]),rowSums(mat[,B,drop=FALSE]),normalizeFun=function(i) i-shift,...)
+      } else {
+        norm.mat=as.matrix(GetTable(data,type=normalization,genes=genes,ntr.na=FALSE))
+        nlfcs=LFC.fun(rowSums(norm.mat[,A,drop=FALSE]),rowSums(norm.mat[,B,drop=FALSE]),normalizeFun=function(i) i)
+        med.element=median(nlfcs)
+        lfcs=LFC.fun(rowSums(mat[,A,drop=FALSE]),rowSums(mat[,B,drop=FALSE]),normalizeFun=function(i) i-med.element,...)
+      }
     } else {
       lfcs=LFC.fun(rowSums(mat[,A,drop=FALSE]),rowSums(mat[,B,drop=FALSE]),...)
     }
-    if (is.data.frame(lfcs)) lfcs else data.frame(LFC=lfcs)
+    lfcs = if (is.data.frame(lfcs)) lfcs else data.frame(LFC=lfcs)
+    if (compute.M) lfcs$M=10^(0.5*(log10(rowSums(mat[,A,drop=FALSE])+0.5)+log10(rowSums(mat[,B,drop=FALSE])+0.5)))
+    lfcs
   })
 }
 
@@ -305,6 +320,7 @@ LFC=function(data, name.prefix = mode, contrasts, slot="count",LFC.fun=lfc::PsiL
 #' @param slot which slot to use (should be a count slot, not normalized values)
 #' @param normalization normalize on "total", "new", or "old" (see details)
 #' @param logFC compute and add the log2 fold change as well
+#' @param genes restrict analysis to these genes; NULL means all genes
 #' @param verbose print status messages?
 #'
 #' @details DESeq2 by default performs size factor normalization. When computing differential expression of new RNA,
@@ -312,16 +328,15 @@ LFC=function(data, name.prefix = mode, contrasts, slot="count",LFC.fun=lfc::PsiL
 #' This can be accomplished by setting mode to "new", and normalization to "total"!
 #'
 #' @details Normalization can also be a mode.slot! Importantly, do not specify a slot containing normalized values, but specify a slot of unnormalized values
-#' (which are used to compute the size factors for normalization!)
+#' (which are used to compute the size factors for normalization!) Can also be a numeric vector of size factors with the same length as the data as columns.
+#' Then each value is divided by the corresponding size factor entry.
 #'
 #' @return a new grandR object including a new analysis table. The columns of the new analysis table are
-#' \itemize{
 #'  \item{"M"}{the base mean}
 #'  \item{"S"}{the log2FoldChange divided by lfcSE}
 #'  \item{"P"}{the Wald test P value}
 #'  \item{"Q"}{same as P but Benjamini-Hochberg multiple testing corrected}
 #'  \item{"LFC"}{the log2 fold change (only with the logFC parameter set to TRUE)}
-#' }
 #'
 #' @seealso \link{LFC},\link{GetContrasts}
 #' @export
@@ -342,7 +357,9 @@ LFC=function(data, name.prefix = mode, contrasts, slot="count",LFC.fun=lfc::PsiL
 PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode="total",
                         slot="count",
                         normalization=NULL,
-                        logFC=FALSE, verbose=FALSE) {
+                        logFC=FALSE,
+                        genes=NULL,
+                        verbose=FALSE) {
   checkPackages("DESeq2")
 
   contrasts = contrasts[,apply(contrasts,2,function(v) all(c(-1,1) %in% v)),drop=FALSE]
@@ -350,18 +367,28 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
 
   mode.slot=paste0(mode,".",slot)
   if (!is.null(normalization)) {
-    if (!check.mode.slot(data,normalization)) normalization=paste0(normalization,".",slot)
+    if (is.numeric(normalization)) {
+      if (length(normalization)!=ncol(data)) stop("Invalid normalization, for size factors make sure that it matches the columns of data!")
+    } else {
+      if (!check.mode.slot(data,normalization)) normalization=paste0(normalization,".",slot)
+      if (!check.mode.slot(data,normalization)) stop("Invalid normalization!")
+    }
   } else {
     normalization = mode.slot
   }
-  if (!check.mode.slot(data,normalization)) stop("Invalid normalization!")
 
   if (!check.mode.slot(data,mode.slot)) stop("Invalid mode")
 
   if (separate) {
-    ApplyContrasts(data,name.prefix=name.prefix,contrasts=contrasts,mode.slot=mode.slot,verbose=verbose,analysis="DESeq2.Wald",FUN=function(mat,A,B) {
-      norm.mat=as.matrix(GetTable(data,type=normalization,ntr.na=FALSE))
-      norm.mat=cbind(norm.mat[,A,drop=FALSE],norm.mat[,B,drop=FALSE])
+    ApplyContrasts(data,name.prefix=name.prefix,contrasts=contrasts,mode.slot=mode.slot,genes=genes,verbose=verbose,analysis="DESeq2.Wald",FUN=function(mat,A,B) {
+      if (is.numeric(normalization)) {
+        nsf = normalization[A|B]
+      } else {
+        norm.mat=as.matrix(GetTable(data,type=normalization,genes=genes,ntr.na=FALSE))
+        norm.mat=cbind(norm.mat[,A,drop=FALSE],norm.mat[,B,drop=FALSE])
+        nsf=DESeq2::estimateSizeFactorsForMatrix(norm.mat)
+      }
+
       A=mat[,A,drop=FALSE]
       B=mat[,B,drop=FALSE]
       coldata=data.frame(comparison=c(rep("A",ncol(A)),rep("B",ncol(B))))
@@ -369,7 +396,7 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
       dds <- DESeq2::DESeqDataSetFromMatrix(countData = cnt(cbind(A,B)),
                                     colData = coldata,
                                     design= ~ comparison-1)
-      DESeq2::sizeFactors(dds)<-DESeq2::estimateSizeFactorsForMatrix(norm.mat)
+      DESeq2::sizeFactors(dds)<-nsf
       l=DESeq2::results(DESeq2::DESeq(dds,quiet=!verbose))
       re=data.frame(
         M=l$baseMean,
@@ -405,10 +432,17 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
       cond.vec[c==1]=A
       cond.vec[c==-1]=B
     }
-    mat=as.matrix(GetTable(data,type=mode.slot,ntr.na=FALSE))
+    mat=as.matrix(GetTable(data,type=mode.slot,ntr.na=FALSE,genes=genes))
     mat=mat[,!is.na(cond.vec)]
-    norm.mat=as.matrix(GetTable(data,type=normalization,ntr.na=FALSE))
-    norm.mat=norm.mat[,!is.na(cond.vec)]
+
+    if (is.numeric(normalization)) {
+      nsf = normalization[!is.na(cond.vec)]
+    } else {
+      norm.mat=as.matrix(GetTable(data,type=normalization,ntr.na=FALSE))
+      norm.mat=norm.mat[,!is.na(cond.vec)]
+      nsf=DESeq2::estimateSizeFactorsForMatrix(norm.mat)
+    }
+
 
     cond.vec=cond.vec[!is.na(cond.vec)]
     coldata=data.frame(comparisons=factor(cond.vec,levels=as.character(1:length(groups))))
@@ -418,7 +452,7 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
                                   colData = coldata,
                                   design= ~ comparisons-1)
 
-    DESeq2::sizeFactors(dds)<-DESeq2::estimateSizeFactorsForMatrix(norm.mat)
+    DESeq2::sizeFactors(dds)<-nsf
     dds <- DESeq2::DESeq(dds,quiet=!verbose)
 
     for (i in 1:length(contrasts)) {
@@ -431,8 +465,8 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
         Q=l$padj
       )
       if (logFC) re.df$LFC=l$log2FoldChange
-      rownames(re.df)<-rownames(data)
-      data=AddAnalysis(data,name = if (is.null(name.prefix)) n else paste0(name.prefix,".",n),table = re.df)
+      rownames(re.df)<-Genes(data,genes=genes,use.symbols = FALSE)
+      data=AddAnalysis(data,name = if (is.null(name.prefix)) n else paste0(name.prefix,".",n),table = re.df,warn.genes = is.null(genes))
     }
     return(data)
   }
@@ -479,7 +513,6 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
 #' @details if N is set to 0, then no sampling from the posterior is performed, but the transformed MAP estimates are returned
 #'
 #' @return a new grandR object including a new analysis table. The columns of the new analysis table are
-#' \itemize{
 #'  \item{"s.A"}{the posterior mean synthesis rate for sample A from the comparison}
 #'  \item{"s.B"}{the posterior mean synthesis rate for sample B from the comparison}
 #'  \item{"HL.A"}{the posterior mean RNA half-life for sample A from the comparison}
@@ -492,7 +525,6 @@ PairwiseDESeq2=function(data, name.prefix=mode, contrasts, separate=FALSE, mode=
 #'  \item{"HL.cred.lower"}{the lower CI boundary of the half-life log2 fold change}
 #'  \item{"HL.cred.upper"}{the upper CI boundary of the half-life log2 fold change}
 #'  \item{"HL.ROPE"}{the signed ROPE probability (negative means downregulation) for the half-life fold change}
-#' }
 #'
 #'
 #' @seealso \link{FitKineticsGeneSnapshot},\link{FitKineticsSnapshot}
@@ -932,11 +964,11 @@ GetSummarizeMatrix.default=function(x,subset=NULL,average=TRUE,...) {
 	rownames(re)=names(v)
 	colnames(re)=unique(v)
 	if (!is.null(subset)) {
-  	save=re[subset,]
+  	save=re[subset,,drop=FALSE]
   	re[,]=0
   	re[subset,]=save
 	}
-	re=re[,colSums(re)>0]
+	re=re[,colSums(re)>0,drop=FALSE]
 	if (average) re=t(t(re)/colSums(re))
 	re
 }
@@ -1053,8 +1085,8 @@ GetContrasts.default=function(x,contrast,columns=NULL,group=NULL,name.format=NUL
       if (is.null(columns)) columns=TRUE
       ll=if (is.factor(coldata[,contrast[1]])) levels(droplevels(coldata[columns,contrast[1]])) else unique(coldata[columns,contrast[1]])
       if (length(ll)<2) stop("Less than 2 levels in contrast: ")
-      re=combn(ll,2,FUN=function(v) make.col(c(contrast,v),group,use)[,1])
-      colnames(re)=combn(ll,2,FUN=function(v) names(make.col(c(contrast,v),group,use))[1])
+      re=utils::combn(ll,2,FUN=function(v) make.col(c(contrast,v),group,use)[,1])
+      colnames(re)=utils::combn(ll,2,FUN=function(v) names(make.col(c(contrast,v),group,use))[1])
       as.data.frame(re,check.names=FALSE)
     }
   }
@@ -1078,6 +1110,53 @@ GetContrasts.default=function(x,contrast,columns=NULL,group=NULL,name.format=NUL
   re
 }
 
+
+#' Create a contrast matrix for two given conditions
+#'
+#' Each column of a contrast matrix represents a pairwise comparison of all samples or cells of
+#' a grandR object (or a column annotation table). Elements being 1 are contrasted vs. elements being -1
+#' (and all 0 are irrelevant for this comparison).
+#'
+#' @param d A grandR object or a column annotation table
+#' @param name the name of the contrast
+#' @param A definition of the condition of interest, see details
+#' @param B definition of the reference condition , see details
+#'
+#' @details This creates a contrast A vs B (i.e. the fold change would be A/B)
+#'
+#' @details Columns that belong to A or B can be given as a logical, integer or character vector representing a selection of the columns (samples or cells).
+#' The expression is evaluated in an environment having the \code{\link{Coldata}}, i.e. you can use names of \code{\link{Coldata}} as variables to
+#' conveniently build a logical vector (e.g., columns=Condition="x").
+#'
+#' @return A data frame with a single column representig a contrast matrix to be used in \code{\link{ApplyContrasts}}, \code{\link{LFC}}, \code{\link{PairwiseDESeq2}}
+#'
+#' @seealso \code{\link{ApplyContrasts}}, \code{\link{LFC}}, \code{\link{PairwiseDESeq2}}, \code{\link{GetContrasts}}
+#'
+#' @export
+#'
+#' @concept diffexp
+GetPairContrasts=function(d,name,A,B) {
+
+  A=substitute(A)
+  B=substitute(B)
+  A=if (is.null(A)) Columns(d) else eval(A,Coldata(d),parent.frame())
+  B=if (is.null(B)) Columns(d) else eval(B,Coldata(d),parent.frame())
+  A=Columns(d,A)
+  B=Columns(d,B)
+
+  if (length(intersect(A,B))>0) stop("Must be mutually exclusive!")
+
+  re = matrix(0,nrow = ncol(d),ncol = 1)
+  rownames(re) = colnames(d)
+  colnames(re) = name
+
+  re[A,1] = 1
+  re[B,1] = -1
+
+  as.data.frame(re)
+}
+
+
 # Normalization of NTRs such that: median logFC new RNA vs. new RNA is 0, there is no correlation of this logFC vs the NTR
 NormalizeEffectiveLabeling=function(data,reference=colnames(data),slot="norm",verbose=FALSE) {
   checkPackages("quantreg")
