@@ -1195,14 +1195,12 @@ CalibrateEffectiveLabelingTimeMatchHalflives=function(data,reference.halflives=N
 #' @details if N is set to 0, then no sampling from the posterior is performed, but the transformed MAP estimates are returned
 #'
 #' @return a new grandR object including new analysis tables (one per condition). The columns of the new analysis table are
-#' \itemize{
 #'  \item{"s"}{the posterior mean synthesis rate}
 #'  \item{"HL"}{the posterior mean RNA half-life}
 #'  \item{"s.cred.lower"}{the lower CI boundary of the synthesis rate}
 #'  \item{"s.cred.upper"}{the upper CI boundary of the synthesis rate}
 #'  \item{"HL.cred.lower"}{the lower CI boundary of the half-life}
 #'  \item{"HL.cred.upper"}{the upper CI boundary of the half-life}
-#' }
 #'
 #'
 #' @export
@@ -1220,6 +1218,7 @@ FitKineticsSnapshot=function(data,name.prefix="Kinetics",reference.columns=NULL,
   }
 
   if (is.null(conditions)) conditions=levels(Condition(data))
+  original.dispersion = dispersion
 
   for (n in conditions) {
     if (verbose) cat(sprintf("Computing snapshot kinetics for %s...\n",n))
@@ -1228,7 +1227,7 @@ FitKineticsSnapshot=function(data,name.prefix="Kinetics",reference.columns=NULL,
     if (is.null(reference.columns)) ss=A
     if (is.matrix(reference.columns)) ss=apply(reference.columns[,Columns(data,A),drop=FALSE]==1,1,any)
     if (length(Columns(data,ss))==0) stop("No reference columns found; check your reference.columns parameter!")
-    dispersion = if (sum(ss)==1) rep(0.1,nrow(data)) else if (!is.null(dispersion)) rep(dispersion,length.out=nrow(data)) else estimate.dispersion(GetTable(data,type="count",columns = ss))
+    dispersion = if (sum(ss)==1) rep(0.1,nrow(data)) else if (!is.null(original.dispersion)) rep(original.dispersion,length.out=nrow(data)) else estimate.dispersion(GetTable(data,type="count",columns = ss))
     if (verbose) {
       if (any(ss & A)) {
         cat(sprintf("Sampling from steady state for %s...\n",paste(colnames(data)[A],collapse = ",")))
@@ -1588,7 +1587,7 @@ TransformSnapshot=function(ntr,total,t,t0=NULL,f0=NULL,full.return=FALSE) {
 #' @param gene the gene to be plotted
 #' @param slot the data slot of the observed abundances
 #' @param time the labeling duration column in the column annotation table
-#' @param type how to fit the model (see link{FitKinetics})
+#' @param type how to fit the model (see \link{FitKinetics})
 #' @param exact.tics use axis labels directly corresponding to the available labeling durations?
 #' @param show.CI show confidence intervals; one of TRUE/FALSE (default: FALSE)
 #' @param return.tables also return the tables used for plotting
@@ -1758,6 +1757,7 @@ SimulateKinetics=function(s=100*d,d=log(2)/hl,hl=2,f0=NULL,min.time=-1,max.time=
 #' @param old show old RNA?
 #' @param new show new RNA?
 #' @param total show total RNA?
+#' @param line.size which line size to use
 #'
 #' @return a ggplot object
 #'
@@ -1767,7 +1767,7 @@ SimulateKinetics=function(s=100*d,d=log(2)/hl,hl=2,f0=NULL,min.time=-1,max.time=
 #' @examples
 #' PlotSimulation(SimulateKinetics(hl=2))
 #' @concept kinetics
-PlotSimulation=function(sim.df,ntr=TRUE,old=TRUE,new=TRUE,total=TRUE) {
+PlotSimulation=function(sim.df,ntr=TRUE,old=TRUE,new=TRUE,total=TRUE, line.size = 1) {
   # R CMD check guard for non-standard evaluation
   Time <- Value <- Type <- NULL
 
@@ -1778,7 +1778,7 @@ PlotSimulation=function(sim.df,ntr=TRUE,old=TRUE,new=TRUE,total=TRUE) {
     sim.df$Type=droplevels(sim.df$Type)
     ggplot(sim.df,aes(Time,Value,color=Type))+
       cowplot::theme_cowplot()+
-      geom_line(size=1)+
+      geom_line(size=line.size)+
         scale_color_manual(NULL,values=c(Old="#54668d",New="#953f36",Total="#373737",NTR="#e4c534")[levels(sim.df$Type)])+
         facet_wrap(~ifelse(Type=="NTR","NTR","Timecourse"),scales="free_y",ncol=1)+
         ylab(NULL)+
